<?php

namespace Drupal\commerce_hubspot\Command;

use Drupal\hubspot_api\Manager as HubSpotManager;
use Drush\Commands\DrushCommands;

/**
 * Provides Drush commands related to HubSpot CRM Associations.
 */
class Association extends DrushCommands {

  /**
   * The HubSpot API manager.
   *
   * @var \Drupal\hubspot_api\Manager
   */
  protected $hubSpot;

  /**
   * Constructs a new Association instance.
   *
   * @param \Drupal\hubspot_api\Manager $hubspot_manager
   *   The HubSpot API manager.
   */
  public function __construct(HubSpotManager $hubspot_manager) {
    $this->hubSpot = $hubspot_manager;
  }

  /**
   * Lists all CRM association types for the given object types.
   *
   * @param string $from_object_type
   *   The type of the origin object of the association.
   * @param string $to_object_type
   *   The type of the target object of the association.
   *
   * @command commerce-hubspot:list-hs-association-types
   *   Fetches and prints the ID/name pairs of all HubSpot association types for
   *   the given from/to object types.
   *
   * @usage commerce-hubspot:list-hs-association-types contacts companies
   *   Fetches and prints the ID/name pairs of all HubSpot association types
   *   from contacts to companies.
   */
  public function listAllTypes(string $from_object_type, $to_object_type) {
    $associations = $this->hubSpot
      ->getHandler()
      ->crm()
      ->associations()
      ->typesApi()
      ->getAll($from_object_type, $to_object_type)
      ->getResults();

    if (!count($associations)) {
      $this->logger->notice(dt(
        'No association types exist for the given origin and target object types.'
      ));
      return;
    }

    foreach ($associations as $association) {
      $this->logger->notice(dt(
        'ID: @id, name: @name',
        [
          '@id' => $association->getId(),
          '@name' => $association->getName(),
        ]
      ));
    }
  }

}
