<?php

namespace Drupal\commerce_hubspot\HubSpot\Search;

use Drupal\commerce_hubspot\HubSpot\Helper as HubSpotHelper;
use Drupal\hubspot_api\Manager as HubSpotManager;

/**
 * The default implementation for HubSpot finders.
 */
class Finder implements FinderInterface {

  /**
   * The model factory for the API being searched.
   *
   * @var \Drupal\commerce_hubspot\HubSpot\Search\ModelFactoryInterface
   */
  protected $modelFactory;

  /**
   * The HubSpot API manager.
   *
   * @var \Drupal\hubspot_api\Manager
   */
  protected $hubSpot;

  /**
   * The ID of the API that the finder is related to.
   *
   * @var string
   */
  protected $apiId;

  /**
   * Constructs a new Finder instance.
   *
   * @param \Drupal\hubspot_api\Manager $hubspot_manager
   *   The HubSpot API manager.
   * @param \Drupal\commerce_hubspot\HubSpot\Search\ModelFactoryInterface $model_factory
   *   The model factory for the API being searched.
   * @param string $api_id
   *   The ID of the API that the finder is related to.
   */
  public function __construct(
    HubSpotManager $hubspot_manager,
    ModelFactoryInterface $model_factory,
    string $api_id
  ) {
    $this->hubSpot = $hubspot_manager;
    $this->modelFactory = $model_factory;
    $this->apiId = $api_id;
  }

  /**
   * Finds the HubSpot record uniquely identified by the given property.
   *
   * HubSpot allows registering properties as unique identifiers so that
   * duplicate records that have the same values for such properties cannot
   * be created. However, they may not be properly registered and we could
   * actually have duplicate records. The results of this search query may be
   * used with the assumption that there is one result only and in order to
   * alter records in subsequent queries. We do not want to just return the
   * first result since it could lead to altering the wrong record. We
   * therefore throw an exception if there is more than one result.
   *
   * @param string $name
   *   The name of the property.
   * @param string $value
   *   The value of the property.
   *
   * @return mixed|null
   *   The result object, or NULL if no HubSpot deal was found.
   *
   * @throws \RuntimeException
   *   When more than one HubSpot record was found with the given property
   *   value.
   */
  public function findByUniqueIdentifier(string $name, string $value) {
    $filter = $this->modelFactory->filter();
    $filter
      ->setOperator('EQ')
      ->setPropertyName($name)
      ->setValue($value);

    $filter_group = $this->modelFactory->filterGroup();
    $filter_group->setFilters([$filter]);

    $search_request = $this->modelFactory->searchRequest();
    $search_request->setFilterGroups([$filter_group]);
    // Only fetch one record to keep the query as efficient as possible. If we
    // have more than one result we'll be throwing an exception.
    $search_request->setLimit(1);

    $client_method = HubSpotHelper::objectTypeToClientMethod($this->apiId);
    $response = $this->hubSpot
      ->getHandler()
      ->crm()
      ->{$client_method}()
      ->searchApi()
      ->doSearch($search_request);

    $total = $response->getTotal();
    if ($total === 0) {
      return NULL;
    }

    if ($total > 1) {
      throw new \RuntimeException(sprintf(
        'More than one HubSpot record was found with value "%s" for property "%s".',
        $value,
        $name
      ));
    }

    return $response->getResults()[0];
  }

}
