<?php

namespace Drupal\commerce_hubspot\HubSpot\Search;

use Drupal\hubspot_api\Manager as HubSpotManager;
use HubSpot\Crm\ObjectType;

/**
 * The default implementation for the HubSpot finder factory.
 *
 * The finder factory facilitates getting a finder for APIs that support
 * searches.
 */
class FinderFactory {

  /**
   * The HubSpot API manager.
   *
   * @var \Drupal\hubspot_api\Manager
   */
  protected $hubSpot;

  /**
   * All instantiated finders keyed by API ID.
   *
   * @var \Drupal\commerce_hubspot\HubSpot\Search\FinderInterface[]
   */
  protected $finders = [];

  /**
   * The class names of the factories for all supported APIs, keyed by API ID.
   *
   * @var array
   */
  protected $factoryMap = [
    ObjectType::COMPANIES => '\Drupal\commerce_hubspot\HubSpot\Search\CompanyModelFactory',
    ObjectType::CONTACTS => '\Drupal\commerce_hubspot\HubSpot\Search\ContactModelFactory',
    ObjectType::DEALS => '\Drupal\commerce_hubspot\HubSpot\Search\DealModelFactory',
    ObjectType::LINE_ITEMS => '\Drupal\commerce_hubspot\HubSpot\Search\LineItemModelFactory',
    ObjectType::PRODUCTS => '\Drupal\commerce_hubspot\HubSpot\Search\ProductModelFactory',
  ];

  /**
   * Constructs a new FinderFactory object.
   *
   * @param \Drupal\hubspot_api\Manager $hubspot_manager
   *   The HubSpot API manager.
   */
  public function __construct(HubSpotManager $hubspot_manager) {
    $this->hubSpot = $hubspot_manager;
  }

  /**
   * Returns the finder for the given API ID.
   *
   * @return \Drupal\commerce_hubspot\HubSpot\Search\FinderInterface
   *   The finder for the given API ID.
   *
   * @throws \InvalidArgumentException
   *   When the given API ID does not exist or is not supported.
   */
  public function get(string $api_id) {
    if (!array_key_exists($api_id, $this->factoryMap)) {
      throw new \InvalidArgumentException(sprintf(
        'Unknown or unsupported API ID "%s".',
        $api_id
      ));
    }

    if (array_key_exists($api_id, $this->finders)) {
      return $this->finders[$api_id];
    }

    return new Finder(
      $this->hubSpot,
      new $this->factoryMap[$api_id](),
      $api_id
    );
  }

}
